local _G = _G
local date = _G.date
local MAJOR_VERSION = 'GraveLevelBar'
local MINOR_VERSION = type('20110304180250') == 'number' and '20110304180250' or date('%Y%m%d%H%M%S')
local MODULE_NAME = 'GuildXP'

local LibStub = _G.LibStub
local AceAddon = LibStub('AceAddon-3.0')
local addon = AceAddon:GetAddon(MAJOR_VERSION)

local module = addon:createXPModule(MODULE_NAME, 'AceEvent-3.0')

local UnitGetGuildXP = _G.UnitGetGuildXP
local GetGuildLevel  = _G.GetGuildLevel
local QueryGuildXP   = _G.QueryGuildXP
local min = min

-- start xp
function module:getStartXPMin()
    return 0
end

-- target level
function module:getTargetLevelMax()
    return MAX_GUILD_LEVEL
end

module.oldEnable = module.OnEnable
function module:OnEnable()
    self:oldEnable()
    self:RegisterEvent('GUILD_XP_UPDATE','Invalidate')
    -- TODO find proper events

    QueryGuildXP()
end

function module:Redraw()
    --addon:Debug("Guild XP Redraw")

    local xpOptions = self:getOptions()
    local barCache = self.cache

    local guildLevel = GetGuildLevel('player')
    local currentGlobalXP = self:getCurrentXP()
    -- update max settings if necessary
    if xpOptions.targetXP and currentGlobalXP > xpOptions.targetXP then
        xpOptions.targetXP = currentGlobalXP
    end
    local startGlobalXP = xpOptions.startXP
    local targetGlobalXP = xpOptions.targetXP or self:calcXpForLevel(MAX_GUILD_LEVEL)
    local remainingGlobalXP = self:getRemainingDailyXP()
    local currentGlobalXPRatio =   (currentGlobalXP  -startGlobalXP)/(targetGlobalXP-startGlobalXP)
    local remainingGlobalXPRatio = (remainingGlobalXP              )/(targetGlobalXP-startGlobalXP)
    -- create textures
    assert(barCache and barCache.frame)
    local frame = barCache.frame
    local right = frame:GetRight()
    local left = frame:GetLeft()
    if not (right and left) then
        self:Invalidate()
        return
    end
    local width = frame:GetRight() - frame:GetLeft()
    -- texture for curent xp
    barCache.currentXPTexture = barCache.currentXPTexture or frame:CreateTexture(nil,'ARTWORK')
    local currentXPTexture = barCache.currentXPTexture
    currentXPTexture:ClearAllPoints()
    currentXPTexture:SetPoint('TOPLEFT')
    currentXPTexture:SetPoint('BOTTOM')
    currentXPTexture:SetWidth(currentGlobalXPRatio * width)
    currentXPTexture:SetTexture(0,0,1,1)

    -- texture for remaining xp
    barCache.remainingXPTexture = barCache.remainingXPTexture or frame:CreateTexture(nil,'ARTWORK')
    local remainingXPTexture = barCache.remainingXPTexture
    remainingXPTexture:ClearAllPoints()
    remainingXPTexture:SetPoint('TOP')
    remainingXPTexture:SetPoint('BOTTOM')
    remainingXPTexture:SetPoint('LEFT', currentXPTexture, 'RIGHT')
    local restedWidth = min(remainingGlobalXPRatio,1-currentGlobalXPRatio) * width
    remainingXPTexture:SetWidth(restedWidth)
    remainingXPTexture:SetTexture(0,0.5,0,1)

    -- textures and numbers for levels
    barCache.levelTextures = barCache.levelTextures or {}
    barCache.levelStrings = barCache.levelStrings or {}
    local levelTextures = barCache.levelTextures
    local levelStrings = barCache.levelStrings
    for level = 1,MAX_GUILD_LEVEL do
        local xpForLeveL = self:calcXpForLevel(level)
        if startGlobalXP <= xpForLeveL and xpForLeveL <= targetGlobalXP then
            local levelGlobalXPRatio = (xpForLeveL-startGlobalXP)/(targetGlobalXP-startGlobalXP)
            -- textures
            levelTextures[level] = levelTextures[level] or frame:CreateTexture(frame:GetName()..'Level'..level,'OVERLAY')
            local levelTexture = levelTextures[level]
            levelTexture:ClearAllPoints()
            levelTexture:SetPoint('TOP')
            levelTexture:SetPoint('BOTTOM')
            if levelGlobalXPRatio < 1.0 then
                levelTexture:SetPoint('LEFT',frame,'LEFT',levelGlobalXPRatio * width,0)
            else
                levelTexture:SetPoint('RIGHT')
            end
            levelTexture:SetWidth(1)
            levelTexture:SetTexture(1,1,1,1)
            levelTexture:Show()
            -- numbers
            levelStrings[level] = levelStrings[level] or frame:CreateFontString(nil,OVERLAY)
            local levelString = levelStrings[level]
            local isValid = levelString:SetFont('Fonts\\FRIZQT__.TTF',xpOptions.size)
            assert(isValid)
            levelString:ClearAllPoints()
            if levelGlobalXPRatio > 0 then
                levelString:SetPoint('RIGHT',levelTexture,'LEFT')
                levelString:SetJustifyH('RIGHT')
            else
                levelString:SetPoint('LEFT',levelTexture,'RIGHT')
                levelString:SetJustifyH('LEFT')
            end
            levelString:SetJustifyV('MIDDLE')
            levelString:SetText(level)
            levelString:Show()
        else
            if levelTextures[level] then
                levelTextures[level]:Hide()
            end
            if levelStrings[level] then
                levelStrings[level]:Hide()
            end
        end
    end

    --update options table
    self:updateOptionsTable()
end

function module:getCurrentXP()
    local xpInCurrentLevel, _, _, _ = UnitGetGuildXP('player')
    local level = GetGuildLevel('player')
    return xpInCurrentLevel + self:calcXpForLevel(level)
end

function module:getRemainingDailyXP()
    local _, _, dailyXP, maxDailyXP = UnitGetGuildXP('player')
    return maxDailyXP - dailyXP
end

function module:calcXpForLevel(level)
    assert(level <= MAX_GUILD_LEVEL,"level=" .. tostring(level))
    local sum = 0
    for i=1,level-1 do
        sum = sum + XP_PER_GUILD_LEVEL[i]
    end
    return sum
end

function module:calcLevelForXP(xp)
    local level = 1
    while xp > self:calcXpForLevel(level+1) do
        level = level + 1
    end
    local startXP = self:calcXpForLevel(level)
    assert(startXP <= xp,"level=" .. level .. " ,xp=" .. xp .. " ,startXP=" .. startXP)
    if startXP == xp then
        return level
    end
    local nextXP = self:calcXpForLevel(level+1)
    local fraction = (xp-startXP)/(nextXP-startXP)
    assert(0.0 <= fraction and fraction <= 1.0, "fraction=" .. fraction .. ", level=" .. level .. " ,xp=" .. xp .. " ,startXP=" .. startXP)
    return level + fraction
end

